# [1.2.0] - 2020-04-30

## Added

- [ISSUE-363](https://github.com/dailymotion/tartiflette/issues/363) - Add an optional `query_cache_decorator` argument at engine initialisation allowing to forward a custom decorator to use to cache query parsing.
- [ISSUE-362](https://github.com/dailymotion/tartiflette/issues/362) - Add an optional `json_loader` argument to engine creation APIs so json loader can be customized.
- [ISSUE-361](https://github.com/dailymotion/tartiflette/issues/361) - Add an optional `custom_default_arguments_coercer` argument at engine initialisation to override the callable used to coerce arguments.
- [ISSUE-361](https://github.com/dailymotion/tartiflette/issues/361) - Add an optional `arguments_coercer` to `@Directive`, `@Subscription` & `@Resolver` decorator to override the callable used to coerce arguments on the decorated directive/field.

## Changed

- [ISSUE-356](https://github.com/dailymotion/tartiflette/issues/356) - Removed dependencies on `flex` and `bison` for installing Tartiflette. `cmake` is still necessary.
- [ISSUE-365](https://github.com/dailymotion/tartiflette/issues/365) - Forward the `InputValueDefinitionNode` to the `on_argument_execution` hook.
  > Note: this introduces a breaking change from previous versions, to upgrade
  > to this version you'll have to update your `on_argument_execution` methods:
  ````patch
  @Directive("MyDirective")
  class MyDirective:
      async def on_argument_execution(
          self,
          directive_args: Dict[str, Any],
          next_directive: Callable,
          parent_node: Union["FieldNode", "DirectiveNode"],
  +       argument_definition_node: "InputValueDefinitionNode",
  -       argument_node: "ArgumentNode",
  +       argument_node: Optional["ArgumentNode"],
          value: Any,
          ctx: Optional[Any],
      ) -> Any:
          # Write your business logic here
  -       return next_directive(parent_node, argument_node, value, ctx)
  +       return next_directive(parent_node, argument_definition_node, argument_node, value, ctx)
  ````
- A list of non-scalar objects in the output will have it's items resolved sequentially, rather than in parallel.

## Fixed

- [ISSUE-370](https://github.com/dailymotion/tartiflette/issues/370) - Fix EnumValue uniqueness in schema definition validation rule. It should now throw the correct error in the correct case.

    ```graphql
    enum anEnum {
        A
        A
        B
    }
    ```

    Will throw a `GraphQLSchemaError` exception at `engine` build time. You can't have duplicates values.

    But now:
    ```graphql
    type X {
        afield:String
    }

    enum anEnum {
        Value1
        X
    }
    ```
    Doesn't throw a `GraphQLSchemaError` for the use of `X` as an `EnumValue`. This was a buggy schema error detection
- [ISSUE-372](https://github.com/dailymotion/tartiflette/issues/372) - Fix SDL Validation, Now `ObjectFollowInterface` validator validate field arguments and allows for field type to be `covariant` of the interface defined type.
- Typing on the documentation related to the `argument_node` argument on the `on_argument_execution` directive hook.
